@@ -1,5 +1,4 @@
-from redis import Redis, ConnectionPool
-from copy import deepcopy
+from redis import StrictRedis, Redis, ConnectionPool
 import math
 from itertools import izip_longest
 
@@ -9,7 +8,7 @@ def grouper(n, iterable, fillvalue=None):
   return izip_longest(fillvalue=fillvalue, *args)
 
 class Leaderboard(object):
-  VERSION = '2.2.2'
+  VERSION = '2.3.0'
   DEFAULT_PAGE_SIZE = 25
   DEFAULT_REDIS_HOST = 'localhost'
   DEFAULT_REDIS_PORT = 6379
@@ -46,7 +45,7 @@ class Leaderboard(object):
     connection_pool : redis connection pool to use if creating a new handle
     '''
     self.leaderboard_name = leaderboard_name
-    self.options = deepcopy(options)
+    self.options = options
 
     self.page_size = self.options.pop('page_size', self.DEFAULT_PAGE_SIZE)
     if self.page_size < 1:
@@ -56,15 +55,16 @@ class Leaderboard(object):
     if not self.order in [self.ASC, self.DESC]:
       raise ValueError("%s is not one of [%s]" % (self.order,  ",".join([self.ASC, self.DESC])))
 
-    self.redis_connection = self.options.pop('connection', None)
-    if not isinstance(self.redis_connection, Redis):
-      if 'connection_pool' not in self.options:
-        self.options['connection_pool'] = self.pool(
-          self.options.pop('host', self.DEFAULT_REDIS_HOST),
-          self.options.pop('port', self.DEFAULT_REDIS_PORT),
-          self.options.pop('db', self.DEFAULT_REDIS_DB)
-        )
-      self.redis_connection = Redis(**self.options)
+    connection = self.options.pop('connection', None)
+    if isinstance(connection, (StrictRedis, Redis)):
+      self.options['connection_pool'] = connection.connection_pool
+    if 'connection_pool' not in self.options:
+      self.options['connection_pool'] = self.pool(
+        self.options.pop('host', self.DEFAULT_REDIS_HOST),
+        self.options.pop('port', self.DEFAULT_REDIS_PORT),
+        self.options.pop('db', self.DEFAULT_REDIS_DB)
+      )
+    self.redis_connection = Redis(**self.options)
 
   def delete_leaderboard(self):
     '''
@@ -419,10 +419,10 @@ class Leaderboard(object):
     @return the page where a member falls in the leaderboard.
     '''
     rank_for_member = None
-    if self.order == self.DESC:
+    if self.order == self.ASC:
       rank_for_member = self.redis_connection.zrank(leaderboard_name, member)
     else:
-      rank_for_memebr = self.redis_connection.zrevrank(leaderboard_name, member)
+      rank_for_member = self.redis_connection.zrevrank(leaderboard_name, member)
 
     if rank_for_member == None:
       rank_for_member = 0
@@ -523,7 +523,6 @@ class Leaderboard(object):
       current_page = 1
 
     page_size = options.get('page_size', self.page_size)
-    total_pages = self.total_pages(page_size = page_size)
 
     index_for_redis = current_page - 1
 
@@ -534,10 +533,7 @@ class Leaderboard(object):
     ending_offset = (starting_offset + page_size) - 1
 
     raw_leader_data = self._range_method(self.redis_connection, self.leaderboard_name, int(starting_offset), int(ending_offset), withscores = False)
-    if raw_leader_data:
-      return self.ranked_in_list_in(self.leaderboard_name, raw_leader_data, **options)
-    else:
-      return None
+    return self._parse_raw_members(self.leaderboard_name, raw_leader_data, **options)
 
   def all_leaders(self, **options):
     '''
@@ -555,11 +551,7 @@ class Leaderboard(object):
     @return the named leaderboard.
     '''
     raw_leader_data = self._range_method(self.redis_connection, leaderboard_name, 0, -1, withscores = False)
-
-    if raw_leader_data:
-      return self.ranked_in_list_in(leaderboard_name, raw_leader_data, **options)
-    else:
-      return []
+    return self._parse_raw_members(leaderboard_name, raw_leader_data, **options)
 
   def members_from_score_range(self, minimum_score, maximum_score, **options):
     '''
@@ -585,11 +577,7 @@ class Leaderboard(object):
       raw_leader_data = self.redis_connection.zrevrangebyscore(leaderboard_name, maximum_score, minimum_score)
     else:
       raw_leader_data = self.redis_connection.zrangebyscore(leaderboard_name, minimum_score, maximum_score)
-
-    if raw_leader_data:
-      return self.ranked_in_list_in(leaderboard_name, raw_leader_data, **options)
-    else:
-      return []
+    return self._parse_raw_members(leaderboard_name, raw_leader_data, **options)
 
   def members_from_rank_range(self, starting_rank, ending_rank, **options):
     '''
@@ -624,10 +612,7 @@ class Leaderboard(object):
     else:
       raw_leader_data = self.redis_connection.zrange(leaderboard_name, starting_rank, ending_rank, withscores = False)
 
-    if raw_leader_data:
-      return self. ranked_in_list_in(leaderboard_name, raw_leader_data, **options)
-    else:
-      return []
+    return self._parse_raw_members(leaderboard_name, raw_leader_data, **options)
 
   def member_at(self, position, **options):
     '''
@@ -690,11 +675,7 @@ class Leaderboard(object):
     ending_offset = (starting_offset + page_size) - 1
 
     raw_leader_data = self._range_method(self.redis_connection, self.leaderboard_name, int(starting_offset), int(ending_offset), withscores = False)
-
-    if raw_leader_data:
-      return self.ranked_in_list_in(leaderboard_name, raw_leader_data, **options)
-    else:
-      return []
+    return self._parse_raw_members(leaderboard_name, raw_leader_data, **options)
 
   def ranked_in_list(self, members, **options):
     '''
@@ -785,3 +766,20 @@ class Leaderboard(object):
     @return a key in the form of +leaderboard_name:member_data+
     '''
     return '%s:member_data' % leaderboard_name
+
+  def _parse_raw_members(self, leaderboard_name, members, members_only = False, **options):
+    '''
+    Parse the raw leaders data as returned from a given leader board query. Do associative lookups with the member to rank, score and potentially sort the results.
+    @param leaderboard_name [String] Name of the leaderboard.
+    @param members [List] A list of members as returned from a sorted set range query
+    @param members_only [bool] Set True to return the members as is, Default is False.
+    @param options [Hash] Options to be used when retrieving the page from the named leaderboard.
+    @return a list of members.
+    '''
+    if members_only:
+      return [{'member': m} for m in members]
+
+    if members:
+      return self.ranked_in_list_in(leaderboard_name, members, **options)
+    else:
+      return []
